"use client"

import React from 'react'
import { useDebounce } from '@/hooks/useDebounce';
import { useLoading } from '@/lib/useLoading';
import { deleteToken, getToken, updateToken } from '@/actions/bmv';
import {
    Card,
    CardContent,
    CardDescription,
    CardHeader,
    CardTitle,
} from "@/components/ui/card"
import { toast } from 'sonner';
import AddToken from './add-token';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { CopyIcon, Search, XIcon } from 'lucide-react';
import DataTable from '@/components/table/data-table';
import { ColumnDef } from '@tanstack/react-table';
import DeleteDialog from '@/components/delete/delete-dialog';
import UpdateToken from './update-token';
import { useQuery } from '@tanstack/react-query';
import { toasty } from '@/utils/toasty';

const ListToken = () => {
    const [offset, setOffset] = React.useState(0);
    const [search, setSearch] = React.useState('');
    const debSearch = useDebounce(search, 500);
    const {loading, setLoading} = useLoading();

    const { data, refetch, isPending, isError, error, isLoading} = useQuery({
        queryKey: ['ttd-token', debSearch, offset],
        queryFn: async () => {
            const response = await getToken(offset, debSearch);
            if (!response.success) {
                throw new Error(response.message);
            }
            return response;
        },
        refetchInterval: 60 * 1000,
        refetchOnWindowFocus: false,
    });

    React.useEffect(() => {
        if (isError && error) {
            toast.error(error.message || "Failed to fetch data");
        }

        if (loading && !isLoading) {
            setLoading(false);
        }
    }, [loading, isError, error, isLoading]);

    const tokens = data?.data || [];
    const hasMore = data?.hasMore || false;
    const page = {
        page: data?.currentPage || 1,
        allPage: data?.totalPages || 1,
        total: data?.total || 0,
    };

    const nextPage = () => {
        if (hasMore) setOffset(prev => prev + 10);
    };

    const prevPage = () => {
        setOffset(prev => Math.max(0, prev - 10));
    };

    const clearSearch = () => {
        setSearch('');
        setOffset(0);
    };

    const handleSearchChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        setSearch(e.target.value);
        setOffset(0)
    };

    const columns: ColumnDef<any>[] = [
        {
            accessorKey: 'name',
            header: () => <span className='font-bold'>TOKEN NAME</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return <p className='w-fit'>{row.name}</p>
            },
        },
        {
            accessorKey: 'token',
            header: () => <span className='font-bold'>TOKEN</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center'>
                        <p className='w-fit'>{row.token.slice(0, 50)}...</p>
                        <Button size="sm" variant="ghost" onClick={() => copyToClipboard(row.token)}>
                            <CopyIcon />
                        </Button>
                    </div>
                )
            },
        },
        {
            accessorKey: 'action',
            header: () => <span className='font-bold'>Action</span>,
            enableSorting: false,
            enableColumnFilter: false,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center gap-2'>
                        <UpdateToken
                            onUpdate={() => handleUpdate(row._id, row.name)}
                        />
                        <DeleteDialog 
                            onDelete={() => handleDelete(row._id)}
                        />
                    </div>
                );
            },
        },
    ]

    const handleUpdate = async (id: string, name: string) => {
        if(!id || !name) return;
        toasty(updateToken(id ,{name}),{
            loading() {
                setLoading(true)
                return "Updating token..."
            },
            success(data) {
                refetch()
                return data.message
            },
            error(err) {
                return err.message || "Failed to update token"
            },
            finally() {
                setLoading(false)
            },
        })
    }

    const handleDelete = async (id: string) => {
        if(!id) return;
        toasty(deleteToken(id),{
            loading() {
                setLoading(true)
                return "Deleting token..."
            },
            success(data) {
                refetch()
                return data.message
            },
            error(err) {
                return err.message || "Failed to delete token"
            },
            finally() {
                setLoading(false)
            },
        })
    }

    const copyToClipboard = (text: string) => {
        navigator.clipboard
        .writeText(text)
        .then(() => {
            toast.success("Copied to clipboard!");
        })
        .catch(() => {
            toast.error("Failed to copy!");
        });
    };
    return (
        <div className="grid grid-cols-3 gap-6">
            <Card className='h-fit'>
                <CardHeader>
                    <CardTitle>Generate New Token</CardTitle>
                    <CardDescription>Create a new token to grant API access with specific permissions.</CardDescription>
                </CardHeader>
                <CardContent>
                    <AddToken onAction={() => refetch()}/>
                </CardContent>
            </Card>
            <div className='col-span-2'>
                <Card>
                    <CardHeader>
                        <CardTitle>API Access Tokens</CardTitle>
                        <CardDescription>Manage the list of tokens used to access the API.</CardDescription>
                    </CardHeader>
                    <CardContent className='space-y-6'>
                        <div className='flex items-center justify-between'>
                            <div className='flex items-center gap-3'>
                                <div className="relative max-w-2xs">
                                    <Input
                                        type="text"
                                        value={search}
                                        placeholder="Search token..."
                                        className="border px-2 py-1 rounded-lg"
                                        onChange={handleSearchChange}
                                    />
                                    {search && (
                                        <div className="absolute top-2 right-2 translate translate-y-0">
                                            <Button 
                                                type="button" 
                                                variant="black" 
                                                size="bullet" 
                                                onClick={clearSearch}
                                            >
                                                <XIcon />
                                            </Button>
                                        </div>
                                    )}
                                </div>
                                <Button 
                                    size="rounded"
                                    onClick={() => {
                                        if(!search) return;
                                        refetch()
                                    }}
                                >
                                    <Search />
                                </Button>
                            </div>
                            <div className="flex items-center gap-3">
                                <Button variant="green" onClick={() => refetch()}>
                                    Refresh Data
                                </Button>
                            </div>
                        </div>
                        <DataTable 
                            data={tokens}
                            columns={columns}
                            isLoading={isPending || isLoading}
                            next={nextPage}
                            prev={prevPage}
                            canNextpage={!hasMore}
                            canPrevPage={offset === 0}
                            page={page?.page || 0}
                            allPage={page?.allPage || 0}
                            total={page?.total || 0}
                        />
                    </CardContent>
                </Card>
            </div>
        </div>
    )
}

export default ListToken